#include "printf.h"
#include "as31glue.h"
#include "main.h"
#include "display.h"
#include "params.h"

#include "sta013.h"

#define MAX_ATTN 64

#define STA013_TREBLE 0x7B
#define STA013_BASS 0x7C
#define STA013_TONE_ATTEN 0x7D
#define STA013_MAX_TONE 12
#define STA013_TREBLE_FREQUENCY_LOW     0x77
#define STA013_TREBLE_FREQUENCY_HIGH    0x78
#define STA013_BASS_FREQUENCY_LOW       0x79
#define STA013_BASS_FREQUENCY_HIGH      0x7A
#define STA013_MUTE 0x14


static xdata unsigned int silent_mp3_clip;


void set_sta013_tone_levels(void)
{
	setting.tone_attn = 0;
	if (setting.treble > (char)setting.tone_attn)
		setting.tone_attn = (unsigned char)setting.treble;
	if (setting.bass > (char)setting.tone_attn)
		setting.tone_attn = (unsigned char)setting.bass;

        sta013_write(STA013_TONE_ATTEN, &setting.tone_attn);
        sta013_write(STA013_TREBLE, &setting.treble);
        sta013_write(STA013_BASS, &setting.bass);
}


void restore_sta013_settings(void)
{
        setting.attn = read_param_byte(PARAM_ATTN);
        sta013_set_attenuation(setting.attn);

        //setting.tone_attn = STA013_MAX_TONE;

        setting.treble = read_param_byte(PARAM_TREBLE);
        setting.bass = read_param_byte(PARAM_BASS);
	set_sta013_tone_levels();

        setting.treble_freq_hi = read_param_byte(PARAM_TREBLE_FREQ_HI);
        sta013_write(STA013_TREBLE_FREQUENCY_HIGH, &setting.treble_freq_hi);
        setting.treble_freq_lo = read_param_byte(PARAM_TREBLE_FREQ_LO);
        sta013_write(STA013_TREBLE_FREQUENCY_LOW, &setting.treble_freq_lo);
        setting.bass_freq_hi = read_param_byte(PARAM_BASS_FREQ_HI);
        sta013_write(STA013_BASS_FREQUENCY_HIGH, &setting.bass_freq_hi);
        setting.bass_freq_lo = read_param_byte(PARAM_BASS_FREQ_LO);
        sta013_write(STA013_BASS_FREQUENCY_LOW, &setting.bass_freq_lo);
}


void backup_sta013_settings(void)
{
	ibuf[0]=setting.attn;
	write_flash_param(PARAM_ATTN, ibuf);
	ibuf[0]=setting.treble;
	write_flash_param(PARAM_TREBLE, ibuf);

	ibuf[0]=setting.bass;
	write_flash_param(PARAM_BASS, ibuf);

	ibuf[0]=setting.treble_freq_hi;
	write_flash_param(PARAM_TREBLE_FREQ_HI, ibuf);
	ibuf[0]=setting.treble_freq_lo;
	write_flash_param(PARAM_TREBLE_FREQ_LO, ibuf);

	ibuf[0]=setting.bass_freq_hi;
	write_flash_param(PARAM_BASS_FREQ_HI, ibuf);
	ibuf[0]=setting.bass_freq_lo;
	write_flash_param(PARAM_BASS_FREQ_LO, ibuf);
}



void action_vol_up(void)
//
//      action_vol_up
//
//      Turn up volume. 
//
{
        // clear_timer(2); 
        if(--setting.attn==255) setting.attn=0;
        if(--setting.attn==255) setting.attn=0;
        sta013_set_attenuation(setting.attn);
        lcd_change(LCD_VOLUME);

        update_sta013_needed = 1;
        set_timer(3,100);
}


void action_vol_down(void)
//
//      action_vol_down
//
//      Turn down volume.
//
{
        // clear_timer(2);
        if(++setting.attn>MAX_ATTN) setting.attn=MAX_ATTN;
        if(++setting.attn>MAX_ATTN) setting.attn=MAX_ATTN;
        sta013_set_attenuation(setting.attn);
        lcd_change(LCD_VOLUME);
        
        update_sta013_needed = 1;
        set_timer(3,100);
}


void action_treble_up(void)
//
//      action_treble_up
//
//      Turn up treble.
//
{
        // clear_timer(2);
        if (++setting.treble > STA013_MAX_TONE)
                setting.treble = STA013_MAX_TONE;
	set_sta013_tone_levels();
        //sta013_write(STA013_TREBLE, &setting.treble);
        lcd_change(LCD_TREBLE);

        update_sta013_needed = 1;
        set_timer(3,100);
}





void action_treble_down(void)
//
//      action_treble_down
//
//      Turn down treble.
//
{
        // clear_timer(2);
        if (--setting.treble < STA013_MAX_TONE * -1)
                setting.treble = STA013_MAX_TONE * -1;
	set_sta013_tone_levels();
        //sta013_write(STA013_TREBLE, &setting.treble);
        lcd_change(LCD_TREBLE);

        update_sta013_needed = 1;
        set_timer(3,100);
}


void action_bass_up(void)
//
//      action_bass_up
//
//      Turn up bass.
//
{
        // clear_timer(2);
        if (++setting.bass > STA013_MAX_TONE)
                setting.bass = STA013_MAX_TONE;
	set_sta013_tone_levels();
        //sta013_write(STA013_BASS, &setting.bass);
        lcd_change(LCD_BASS);

        update_sta013_needed = 1;
        set_timer(3,100);
}

void action_bass_down(void)
//
//      action_bass_down
//
//      Turn down bass.
//
{
        // clear_timer(2);
        if (--setting.bass < STA013_MAX_TONE * -1)
                setting.bass = STA013_MAX_TONE * -1;
	set_sta013_tone_levels();
        //sta013_write(STA013_BASS, &setting.bass);
        lcd_change(LCD_BASS);

        update_sta013_needed = 1;
        set_timer(3,100);
}


void action_treble_freq_up(void)
//
//      action_treble_freq_up
//
//      Turn up treble cutoff frequency.
//
{
        int treble_frequency;

        // clear_timer(2);

        treble_frequency=((int)setting.treble_freq_hi<<8)+setting.treble_freq_lo;

        if(treble_frequency<9990)
                treble_frequency+=10;

        setting.treble_freq_hi=(treble_frequency & 0xFF00)>>8;
        setting.treble_freq_lo=(treble_frequency & 0x00FF);
        sta013_write(STA013_TREBLE_FREQUENCY_HIGH, &setting.treble_freq_hi);
        sta013_write(STA013_TREBLE_FREQUENCY_LOW, &setting.treble_freq_lo);
        lcd_change(LCD_TREBLE_FREQ);

        update_sta013_needed = 1;
        set_timer(3,100);
}


void action_treble_freq_down(void)
//
//      action_treble_freq_down
//
//      Turn down treble cutoff frequency.
//
{
        int treble_frequency;

        // clear_timer(2);

        treble_frequency=((int)setting.treble_freq_hi<<8)+setting.treble_freq_lo;

        if(treble_frequency>10)
                treble_frequency-=10;

        setting.treble_freq_hi=(treble_frequency & 0xFF00)>>8;
        setting.treble_freq_lo=(treble_frequency & 0x00FF);
        sta013_write(STA013_TREBLE_FREQUENCY_HIGH, &setting.treble_freq_hi);
        sta013_write(STA013_TREBLE_FREQUENCY_LOW, &setting.treble_freq_lo);
        lcd_change(LCD_TREBLE_FREQ);

        update_sta013_needed = 1;
        set_timer(3,100);
}


void action_bass_freq_up(void)
//
//      action_bass_freq_up
//
//      Turn up bass cutoff frequency.
//
{
        int bass_frequency;

        // clear_timer(2);

        bass_frequency=((int)setting.bass_freq_hi<<8)+setting.bass_freq_lo;

        if(bass_frequency<9990)
                bass_frequency+=10;

        setting.bass_freq_hi=(bass_frequency & 0xFF00)>>8;
        setting.bass_freq_lo=(bass_frequency & 0x00FF);
        sta013_write(STA013_BASS_FREQUENCY_HIGH, &setting.bass_freq_hi);
        sta013_write(STA013_BASS_FREQUENCY_LOW, &setting.bass_freq_lo);
        lcd_change(LCD_BASS_FREQ);

        update_sta013_needed = 1;
        set_timer(3,100);
}


void action_bass_freq_down(void)
//
//      action_bass_freq_down
//
//      Turn down bass cutoff frequency.
//
{
        int bass_frequency;

        // clear_timer(2);

        bass_frequency=((int)setting.bass_freq_hi<<8)+setting.bass_freq_lo;

        if(bass_frequency>10)
                bass_frequency-=10;

        setting.bass_freq_hi=(bass_frequency & 0xFF00)>>8;
        setting.bass_freq_lo=(bass_frequency & 0x00FF);
        sta013_write(STA013_BASS_FREQUENCY_HIGH, &setting.bass_freq_hi);
        sta013_write(STA013_BASS_FREQUENCY_LOW, &setting.bass_freq_lo);
        lcd_change(LCD_BASS_FREQ);

        update_sta013_needed = 1;
        set_timer(3,100);
}










code int mpeg1_bitrates[16]={-1, 32, 40, 48, 56, 64, 80, 96, 112,
	128, 160, 192, 224, 256, 320, -1};
code int mpeg2_bitrates[16]={-1, 8, 16, 24, 32, 40, 48, 56, 64,
	80, 96, 112, 128, 144, 160, -1};
code unsigned char mpeg1_freq[4]={44, 48, 32, 0};
code unsigned char mpeg2_freq[4]={22, 24, 16, 0};
code unsigned char mpeg25_freq[4]={11, 12, 8, 0};

// These are used to get the number of milliseconds per MP3 frame played.
// We'll use this for time display on screen
// BEWARE:  The copies of these numbers in the STA013's docs are inaccurate
// for the 44.1 value.  Check http://www.mpeg.org/ for info on how to calculate
// the proper values (which are included below)/
// -ZSB 12-Aug-2001 2:50 PM
code unsigned char mpeg1_ms_per_frame[4]={26,24,36,0};
code unsigned char mpeg2_ms_per_frame[4]={26,24,36,0};
code unsigned char mpeg25_ms_per_frame[4]={48,48,72,0};

struct playback_struct playback;


static xdata union {
        struct {
                unsigned char l;
                unsigned char m;
                unsigned char h;
                unsigned char sync;
        } byte;
        unsigned long ul;
} header;


unsigned char
update_playback_parms(void) {

	if (playback.mode_known == 0) {
		header.ul = sta013_read_header();

		// TODO: there's a bug in sta013_read_header (or perhaps the
		//  I2C communication code) where every other attempt to read
		//  the header fails.  Uncomment this line to be able to see it.
		// printf("hdr = %lx\r\n", header.ul);
		// I *think* I fixed this -- bug in I2C Code.  -ZSB 

		if (header.byte.sync != 2) return 0;

		if (header.byte.h & 0x08) {     /* mpeg1 stream */
			playback.bitrate = mpeg1_bitrates[header.byte.m >> 4];
			playback.freq = mpeg1_freq[(header.byte.m >> 2) & 3];
			playback.ms_per_frame = mpeg1_ms_per_frame[(header.byte.m >> 2) & 3];
		} else {                /* mpeg2 stream */
			playback.bitrate = mpeg2_bitrates[header.byte.m >> 4];
			if (header.byte.h & 0x10) {     /* mpeg2.0 */
				playback.freq = mpeg2_freq[(header.byte.m >> 2) & 3];
				playback.ms_per_frame = mpeg2_ms_per_frame[(header.byte.m >> 2) & 3];
			} else {                /* mpeg2.5 */
				playback.freq = mpeg25_freq[(header.byte.m >> 2) & 3];
				playback.ms_per_frame = mpeg25_ms_per_frame[(header.byte.m >> 2) & 3];
			}
		}
		
		if ((header.byte.l & 0xC0) == 0xC0) {
			playback.mode = "  Mono";
		} else {
			playback.mode = "Stereo";
		}
		
		playback.mode_known = 1;

		return 1;
	}
	
	return 0;
}

unsigned long GetFrameCount() {
// Returns the current frame count from the STA013 as an unsigned long

	xdata unsigned char tmp;
	xdata unsigned long retval = 0;	
	
	if(!sta013_read(STA013_FRAME_CNT_H, &tmp)) retval += (tmp * 65536);
	if(!sta013_read(STA013_FRAME_CNT_M, &tmp)) retval += (tmp * 256);
	if(!sta013_read(STA013_FRAME_CNT_L, &tmp)) retval += tmp;

	return retval;
}


#define NUM_SILENT_FRAMES 14

void init_silent_mp3_clip(void)
{
	silent_mp3_clip = malloc_blocks(1);
        map_block(silent_mp3_clip, (xdata void *)0x6000);
        fill_null_mp3((xdata void *)0x6000, NUM_SILENT_FRAMES);
}

void play_silent_mp3_clip(void)
{
	print("Playing silent clip\r\n");
	play_block(silent_mp3_clip, NUM_SILENT_FRAMES * 104);
}

/* this should be called right before calling play_abort() */
/* or doing anything else that could disrupt the data flow */
/* to the sta013, so that the user won't hear a blip by */
/* the discontinuity in mpeg data framing.  Once the output */
/* is muted, the silent clip should be played, so that when */
/* new data is sent into the chip, it will have received */
/* several good frames of data and be waiting for the start */
/* of a new frame (instead of the rest of an incomplete one) */

void mute_sta013_output(void)
{
	xdata unsigned char value=1;

	sta013_write(STA013_MUTE, &value);
	mute_is_on = 1;
}



